"""
Derived from the sample code on the developer portal:

https://github.com/ripple/xrpl-dev-portal/blob/57dd03d9a1ff610c12c692ead93a6acb06cfe950/content/_code-samples/tx-serialization/serialize.py
"""

from decimal import getcontext, Decimal
import json
import pkg_resources
import re
import typing as t

import typing_extensions as tex

from xpring.bits import from_bytes, to_bytes
from xpring.codec import DEFAULT_CODEC
from xpring.types import AccountId, Address, Amount, NonXrpAmount, Transaction


def field_id(type_code, field_code):
    """
    Encode a field ID from its type and field codes.

    The encoding occupies 1 to 3 bytes depending on whether the codes are
    "common" (<16) or "uncommon" (>=16).
    """
    # Codes must be nonzero and fit in 1 byte.
    assert 0 < type_code < 256
    assert 0 < field_code < 256

    # Each code takes a nibble (4 bits) in the first byte.
    # If a code spills over the nibble (i.e. >=16), then the nibble is
    # filled with zeroes and the code is given a trailing byte.
    # Type always comes before field.

    if type_code < 16 and field_code < 16:
        return to_bytes(type_code << 4 | field_code, 1)
    if type_code >= 16 and field_code < 16:
        return to_bytes(field_code << 8 | type_code, 2)
    if type_code < 16 and field_code >= 16:
        return to_bytes(type_code << 12 | field_code, 2)
    return to_bytes(type_code << 8 | field_code, 3)


def vl_encode(blob: bytes) -> bytes:
    """
    Encode a variable length type.

    Prefixes an arbitrary byte array (a "blob") with an encoded length. The
    length of the prefix is 1 to 3 bytes depending on the length of the blob:

    <= 918744 bytes: prefix is 3 bytes
    <= 12480 bytes: prefix is 2 bytes
    <= 192 bytes: prefix is 1 byte
    """
    length = len(blob)
    if length > 918744:
        raise ValueError(
            'variable length field must not be longer than 918744 bytes'
        )
    if length > 12480:
        prefix = to_bytes(length - 12481 + (241 << 16), 3)
    elif length > 192:
        prefix = to_bytes(length - 193 + (193 << 8), 2)
    else:
        prefix = bytes([length])
    return prefix + blob


def serialize_account_id(address: str) -> bytes:
    return vl_encode(DEFAULT_CODEC.decode_address(t.cast(Address, address)))


def serialize_amount(amount: Amount) -> bytes:
    """
    Serialize an Amount.

    An XRP Amount comes as a string. It must be serialized to 64 bits:
    1 zero bit, 1 sign bit (zero for negative, one for positive), 62 bits of
    absolute value.

    A non-XRP Amount comes as a dictionary. It must be serialized thus:
    64 bits of unsigned value; 160 bit currency code; 160 bit issuer
    AccountID.
    """
    if isinstance(amount, str):
        value = int(amount)
        sign = int(value > 0)
        magnitude = abs(value)
        assert magnitude <= 10**17
        return to_bytes(sign << 62 | magnitude, 8)
    if isinstance(amount, dict):
        value_bytes = serialize_amount_non_xrp(amount['value'])
        currency_bytes = serialize_currency(amount['currency'])
        address_bytes = DEFAULT_CODEC.decode_address(
            t.cast(Address, amount['issuer'])
        )
        return value_bytes + currency_bytes + address_bytes
    raise ValueError('Amount must be `str` or `{value, currency, issuer}`')


"""
Zero has a special-case canonical format:
- one "not XRP" bit set to 1
- one sign bit set to 0 (for "negative")
- sixty-two value bits set to 0
"""
CANONICAL_ZERO = to_bytes(1 << 63, 8)

MANTISSA_MIN = 10**15
MANTISSA_MAX = 10**16 - 1
EXPONENT_MIN = -96
EXPONENT_MAX = 80


def serialize_amount_non_xrp(value: str) -> bytes:
    context = getcontext()
    context.prec = 15
    context.Emin = EXPONENT_MIN
    context.Emax = EXPONENT_MAX

    decimal = Decimal(value)

    if decimal.is_zero():
        return CANONICAL_ZERO

    # Convert components to integers.
    sign, digits, exponent = decimal.as_tuple()
    mantissa = int(''.join(str(d) for d in digits))

    # Canonicalize to expected range.
    while mantissa < MANTISSA_MIN and exponent > EXPONENT_MIN:
        mantissa *= 10
        exponent -= 1

    while mantissa > MANTISSA_MAX:
        if exponent >= EXPONENT_MAX:
            raise ValueError('amount overflow')
        mantissa //= 10
        exponent += 1

    if exponent < EXPONENT_MIN or mantissa < MANTISSA_MIN:
        # Round to zero.
        return CANONICAL_ZERO

    if exponent > EXPONENT_MAX or mantissa > MANTISSA_MAX:
        raise ValueError('amount overflow')

    # Serialize to bytes.
    bits = 1 << 63  # "not XRP" bit
    if sign == 0:
        bits |= 1 << 62  # "is positive" bit
    bits |= ((exponent + 97) << 54)  # 8 bits of exponent
    bits |= mantissa  # 54 bits of mantissa

    return to_bytes(bits, 8)


def serialize_array(array: t.Iterable) -> bytes:
    """Serialize an array of objects."""
    return b''.join(
        serialize_object(item, terminate=False) for item in array
    ) + ARRAY_END_MARKER


def serialize_blob(blob_hex: str) -> bytes:
    return vl_encode(bytes.fromhex(blob_hex))


# ISO 4217 3-character currency code
CURRENCY_CODE_PATTERN = re.compile(r'^[][A-Za-z0-9?!@#$%^&*<>(){}|]{3}$')
HEX_160_PATTERN = re.compile(r'^[0-9a-fA-F]{40}$')


def serialize_currency(currency: str) -> bytes:
    if CURRENCY_CODE_PATTERN.match(currency):
        if currency == 'XRP':
            # Rare, but when the currency code "XRP" is serialized,
            # it's a special-case: all zeroes.
            return bytes(20)

        code_ascii = currency.encode('ASCII')
        # standard currency codes: https://xrpl.org/currency-formats.html#standard-currency-codes
        # 8 bits type code (0x00)
        # 88 bits reserved (zeroes)
        # 24 bits ASCII
        # 16 bits version (0x0000)
        # 24 bits reserved (zeroes)
        return bytes(12) + code_ascii + bytes(5)

    if HEX_160_PATTERN.match(currency):
        return bytes.fromhex(currency)

    raise ValueError(f'unknown currency code: {currency}')


def serialize_field(field, value):
    id_bytes = field['id']
    serialize = field['serialize']
    if serialize is None:
        field_name = field['name']
        field_type = field['type']
        raise NotImplementedError(
            f'cannot serialize field {field_name} ({field_type})'
        )
    try:
        value_bytes = serialize(value)
    except ValueError as cause:
        field_type = field['type']
        field_name = field['name']
        raise ValueError(f'field {field_name} ({field_type}): {str(cause)}')
    return id_bytes + value_bytes


def serialize_hash(bits: int, digest: str) -> bytes:
    blob = bytes.fromhex(digest)
    if len(blob) * 8 != bits:
        raise ValueError(f'expected {bits} bits: {digest}')
    return blob


def serialize_hash128(digest: str) -> bytes:
    return serialize_hash(128, digest)


def serialize_hash160(digest: str) -> bytes:
    return serialize_hash(160, digest)


def serialize_hash256(digest: str) -> bytes:
    return serialize_hash(256, digest)


def serialize_ledger_entry_type(name: str) -> bytes:
    type_code = LEDGER_ENTRY_TYPES_BY_NAME[name]
    assert type_code >= 0
    return serialize_uint16(type_code)


def field_key(field):
    try:
        return field['key']
    except KeyError as error:
        raise AssertionError(f'field {field_name} missing key')


def serialize_object(
    object_: t.Mapping, signing: bool = False, terminate: bool = True
) -> bytes:
    fields = [FIELDS_BY_NAME[name] for name in object_.keys()]
    fields = [
        field for field in fields
        if field['isSerialized'] and (not signing or field['isSigningField'])
    ]
    fields = sorted(fields, key=field_key)

    blob = bytearray()
    for field in fields:
        blob.extend(serialize_field(field, object_[field['name']]))
    if terminate:
        blob.extend(OBJECT_END_MARKER)
    return bytes(blob)


def serialize_path(path: t.Collection) -> bytes:
    print(f'path: {path}')
    if not len(path):
        raise ValueError('a Path must not be empty')
    return b''.join(serialize_step(step) for step in path)


def serialize_pathset(pathset: t.Collection) -> bytes:
    print(f'pathset: {pathset}')
    if not len(pathset):
        raise ValueError('a PathSet must not be empty')
    return PATH_END_MARKER.join(
        serialize_path(path) for path in pathset
    ) + PATHSET_END_MARKER


def serialize_step(step: t.Mapping) -> bytes:
    blob = bytearray([0])
    if 'account' in step:
        blob[0] |= 0x01
        blob.extend(DEFAULT_CODEC.decode_address(step['account']))
    if 'currency' in step:
        blob[0] |= 0x10
        blob.extend(serialize_currency(step['currency']))
    if 'issuer' in step:
        blob[0] |= 0x20
        blob.extend(DEFAULT_CODEC.decode_address(step['issuer']))
    return bytes(blob)


def serialize_transaction(
    transaction: Transaction, signing: bool = False
) -> bytes:
    return serialize_object(transaction, signing=signing, terminate=False)


def serialize_transaction_type(name: str) -> bytes:
    return to_bytes(TRANSACTION_TYPES_BY_NAME[name], 2)


def serialize_uint(bits: int, value: int) -> bytes:
    assert 0 <= value < (1 << bits)
    return to_bytes(value, bits // 8)


def serialize_uint8(value: int) -> bytes:
    return serialize_uint(8, value)


def serialize_uint16(value: int) -> bytes:
    return serialize_uint(16, value)


def serialize_uint32(value: int) -> bytes:
    return serialize_uint(32, value)


def serialize_uint64(value: str) -> bytes:
    return serialize_hash(64, value)


def serialize_vector256(digests: t.Iterable[str]) -> bytes:
    blob = b''.join(serialize_hash256(digest) for digest in digests)
    return vl_encode(blob)


class Scanner:

    def __init__(self, stream: bytes) -> None:
        self.stream = bytearray(stream)
        self.cursor = 0

    @property
    def inexhausted(self):
        return self.cursor < len(self.stream)

    def __bool__(self):
        return self.inexhausted

    def __len__(self):
        return len(self.stream) - self.cursor

    def extend(self, bite) -> None:
        self.stream.extend(bite)

    def peek(self, length) -> bytes:
        return self.stream[self.cursor:self.cursor + length]

    def bite(self) -> int:
        return self.stream[self.cursor]

    def skip(self, length: int) -> None:
        self.cursor += length

    def take1(self) -> int:
        self.cursor += 1
        return self.stream[self.cursor - 1]

    def take(self, length: int) -> bytes:
        self.cursor += length
        return self.stream[(self.cursor - length):self.cursor]


def vl_decode(scanner: Scanner) -> bytes:
    """Return the next variable-length blob while advancing the cursor.

    https://xrpl.org/serialization.html#length-prefixing
    """
    byte1 = scanner.take1()
    if byte1 < 193:
        length = byte1
    elif byte1 < 241:
        byte2 = scanner.take1()
        length = 193 + (byte1 - 193) * 256 + byte2
    elif byte1 < 255:
        byte2 = scanner.take1()
        byte3 = scanner.take1()
        length = 12481 + (byte1 - 241) * 645536 + byte2 * 256 + byte3
    else:
        raise ValueError(f'not a length prefix: {byte1}')
    return scanner.take(length)


def deserialize_account_id(scanner: Scanner) -> Address:
    account_id = t.cast(AccountId, vl_decode(scanner))
    return DEFAULT_CODEC.encode_address(account_id)


def deserialize_amount(scanner: Scanner) -> Amount:
    byte1 = scanner.bite()
    # Most-significant bit is the format bit. 1 means "is not XRP".
    if not byte1 & (1 << 7):
        # Second most-significant bit is the sign bit. 1 means "is positive".
        sign = 1 if byte1 & (1 << 6) else -1
        # Format bit is already cleared, but clear both top bits regardless.
        magnitude = from_bytes(scanner.take(8)) & ~(0b11 << 62)
        return str(sign * magnitude)
    value = deserialize_amount_non_xrp(scanner)
    currency = deserialize_currency(scanner)
    issuer = DEFAULT_CODEC.encode_address(t.cast(AccountId, scanner.take(20)))
    return {'value': value, 'currency': currency, 'issuer': issuer}


def deserialize_amount_non_xrp(scanner: Scanner) -> str:
    bits = from_bytes(scanner.take(8))
    not_xrp_bit = bits & (1 << 63)
    assert not_xrp_bit
    sign_bit = bits & (1 << 62)
    unsigned_exponent = ((bits >> 54) & 0xFF)
    mantissa = bits & ((1 << 54) - 1)
    if not sign_bit and not unsigned_exponent and not mantissa:
        return '0'
    value = str(mantissa)
    exponent = unsigned_exponent - 97
    if exponent > 0:
        padding = '0' * exponent
        value += padding
    else:
        places = -exponent
        if places > len(value):
            padding = '0' * (places - len(value))
            value = '0.' + padding + value
        else:
            value = value[:exponent] + '.' + value[exponent:]
            if value[0] == '.':
                value = '0' + value
        value = value.rstrip('0')
        if value[-1] == '.':
            value = value[:-1]
    if not sign_bit:
        value = '-' + value
    return value


def deserialize_array(scanner: Scanner) -> t.List:
    array = []
    while scanner.peek(1) != ARRAY_END_MARKER:
        key, value = deserialize_field(scanner)
        array.append({key: value})
    scanner.skip(1)
    return array


def deserialize_blob(scanner: Scanner) -> str:
    return vl_decode(scanner).hex().upper()


def deserialize_currency(scanner: Scanner) -> str:
    blob = scanner.take(160 // 8)
    # If the first 8 bits are 0x00, it is a standard currency code.
    # https://xrpl.org/currency-formats.html#standard-currency-codes
    if blob == bytes(20):
        return 'XRP'
    if not blob[0]:
        return blob[12:15].decode('ASCII')
    # Otherwise, it is a nonstandard currency code.
    return blob.hex().upper()


def deserialize_field(scanner: Scanner) -> t.Tuple[str, t.Any]:
    type_code, field_code = deserialize_field_key(scanner)
    field = FIELDS_BY_ID[(type_code, field_code)]
    deserialize = field['deserialize']
    if deserialize is None:
        field_name = field['name']
        field_type = field['type']
        raise NotImplementedError(
            f'cannot deserialize field {field_name} ({field_type})'
        )
    value = deserialize(scanner)
    return (field['name'], value)


TypeCode = t.NewType('TypeCode', int)
FieldCode = t.NewType('FieldCode', int)


def deserialize_field_key(scanner: Scanner) -> t.Tuple[TypeCode, FieldCode]:
    byte1 = scanner.take1()
    type_code = byte1 >> 4
    if not type_code:
        type_code = scanner.take1()
    field_code = byte1 & 0x0F
    if not field_code:
        field_code = scanner.take1()
    return (t.cast(TypeCode, type_code), t.cast(FieldCode, field_code))


def deserialize_hash(bits: int, scanner: Scanner) -> str:
    return scanner.take(bits // 8).hex().upper()


def deserialize_hash128(scanner: Scanner) -> str:
    return deserialize_hash(128, scanner)


def deserialize_hash160(scanner: Scanner) -> str:
    return deserialize_hash(160, scanner)


def deserialize_hash256(scanner: Scanner) -> str:
    return deserialize_hash(256, scanner)


def deserialize_ledger_entry_type(scanner: Scanner) -> str:
    type_code = deserialize_uint16(scanner)
    return LEDGER_ENTRY_TYPES_BY_CODE[type_code]


def deserialize_object(scanner: Scanner) -> t.Mapping:
    object_ = {}
    while scanner.peek(1) != OBJECT_END_MARKER:
        key, value = deserialize_field(scanner)
        object_[key] = value
    scanner.skip(1)
    return object_


Step = tex.TypedDict(
    'Step', {
        'account': Address,
        'currency': str,
        'issuer': Address,
        'type': int,
        'type_hex': str,
    },
    total=False
)
Path = t.Collection[Step]
PathSet = t.Collection[Path]


def deserialize_path(scanner: Scanner) -> Path:
    path = []
    while scanner.peek(1) not in (PATH_END_MARKER, PATHSET_END_MARKER):
        path.append(deserialize_step(scanner))
    if scanner.peek(1) == PATH_END_MARKER:
        scanner.skip(1)
    return path


def deserialize_pathset(scanner: Scanner) -> PathSet:
    pathset = []
    while scanner.peek(1) != PATHSET_END_MARKER:
        pathset.append(deserialize_path(scanner))
    scanner.skip(1)
    return pathset


def deserialize_step(scanner: Scanner) -> Step:
    type_byte = scanner.take1()
    step = t.cast(Step, {})
    if type_byte & 0x01:
        step['account'] = DEFAULT_CODEC.encode_address(
            t.cast(AccountId, scanner.take(20))
        )
    if type_byte & 0x10:
        step['currency'] = deserialize_currency(scanner)
    if type_byte & 0x20:
        step['issuer'] = DEFAULT_CODEC.encode_address(
            t.cast(AccountId, scanner.take(20))
        )
    step['type'] = type_byte
    step['type_hex'] = to_bytes(type_byte, 8).hex().upper()
    return step


def deserialize_transaction(scanner: Scanner) -> Transaction:
    scanner.extend(OBJECT_END_MARKER)
    return deserialize_object(scanner)


def deserialize_transaction_type(scanner: Scanner) -> str:
    return TRANSACTION_TYPES_BY_CODE[from_bytes(scanner.take(2))]


def deserialize_uint(bits: int, scanner: Scanner) -> int:
    return from_bytes(scanner.take(bits // 8))


def deserialize_uint8(scanner: Scanner) -> int:
    return deserialize_uint(8, scanner)


def deserialize_uint16(scanner: Scanner) -> int:
    return deserialize_uint(16, scanner)


def deserialize_uint32(scanner: Scanner) -> int:
    return deserialize_uint(32, scanner)


def deserialize_uint64(scanner: Scanner) -> str:
    return deserialize_hash(64, scanner)


def deserialize_vector256(scanner: Scanner) -> t.List[str]:
    blob = vl_decode(scanner)
    subscanner = Scanner(blob)
    expected_length = len(subscanner) // (256 // 8)
    digests = []
    while subscanner:
        digests.append(deserialize_hash256(subscanner))
    assert len(digests) == expected_length
    return digests


CODECS = {
    'AccountID': (serialize_account_id, deserialize_account_id),
    'Amount': (serialize_amount, deserialize_amount),
    'Blob': (serialize_blob, deserialize_blob),
    'Hash128': (serialize_hash128, deserialize_hash128),
    'Hash160': (serialize_hash160, deserialize_hash160),
    'Hash256': (serialize_hash256, deserialize_hash256),
    'PathSet': (serialize_pathset, deserialize_pathset),
    'STArray': (serialize_array, deserialize_array),
    'STObject': (serialize_object, deserialize_object),
    'UInt8': (serialize_uint8, deserialize_uint8),
    'UInt16': (serialize_uint16, deserialize_uint16),
    'UInt32': (serialize_uint32, deserialize_uint32),
    'UInt64': (serialize_uint64, deserialize_uint64),
    'Vector256': (serialize_vector256, deserialize_vector256),
}

# TODO: Consider lazy initialization.
_DEFINITIONS = json.load(
    pkg_resources.resource_stream('xpring', 'definitions.json')
)
LEDGER_ENTRY_TYPES_BY_NAME = _DEFINITIONS['LEDGER_ENTRY_TYPES']
LEDGER_ENTRY_TYPES_BY_CODE = {
    v: k for k, v in LEDGER_ENTRY_TYPES_BY_NAME.items()
}
TRANSACTION_TYPES_BY_NAME = _DEFINITIONS['TRANSACTION_TYPES']
TRANSACTION_TYPES_BY_CODE = {v: k for k, v in TRANSACTION_TYPES_BY_NAME.items()}
TYPES_BY_NAME = _DEFINITIONS['TYPES']
TYPES_BY_CODE = {v: k for (k, v) in TYPES_BY_NAME.items()}
FIELDS_BY_NAME = {k: v for (k, v) in _DEFINITIONS['FIELDS']}
for field_name, field in FIELDS_BY_NAME.items():
    type_name = field['type']
    type_code = TYPES_BY_NAME[type_name]
    field_code = field['nth']
    field['name'] = field_name
    if field['isSerialized']:
        assert 0 < type_code < 256 and 0 < field_code < 256
        field['key'] = (type_code, field_code)
        field['id'] = field_id(type_code, field_code)
        field['serialize'], field['deserialize'] = CODECS[type_name]
FIELDS_BY_NAME['TransactionType']['serialize'] = serialize_transaction_type
FIELDS_BY_NAME['TransactionType']['deserialize'] = deserialize_transaction_type
FIELDS_BY_NAME['LedgerEntryType']['serialize'] = serialize_ledger_entry_type
FIELDS_BY_NAME['LedgerEntryType']['deserialize'] = deserialize_ledger_entry_type
FIELDS_BY_ID = {v['key']: v for v in FIELDS_BY_NAME.values() if 'key' in v}
PATH_END_MARKER = b'\xFF'
PATHSET_END_MARKER = b'\x00'
ARRAY_END_MARKER = FIELDS_BY_NAME['ArrayEndMarker']['id']
OBJECT_END_MARKER = FIELDS_BY_NAME['ObjectEndMarker']['id']

# https://github.com/ripple/rippled/blob/develop/src/ripple/protocol/impl/HashPrefix.cpp
PREFIX_TRANSACTION_ID = b'TXN\x00'
PREFIX_TRANSACTION_SIGNATURE = b'STX\x00'



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: org/xrpl/rpc/v1/get_fee.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import amount_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='org/xrpl/rpc/v1/get_fee.proto',
  package='org.xrpl.rpc.v1',
  syntax='proto3',
  serialized_options=b'\n\017org.xrpl.rpc.v1P\001',
  serialized_pb=b'\n\x1dorg/xrpl/rpc/v1/get_fee.proto\x12\x0forg.xrpl.rpc.v1\x1a\x1corg/xrpl/rpc/v1/amount.proto\"\x0f\n\rGetFeeRequest\"\xfc\x01\n\x0eGetFeeResponse\x12\x1f\n\x13\x63urrent_ledger_size\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\x1e\n\x12\x63urrent_queue_size\x18\x02 \x01(\x04\x42\x02\x30\x01\x12!\n\x03\x66\x65\x65\x18\x03 \x01(\x0b\x32\x14.org.xrpl.rpc.v1.Fee\x12 \n\x14\x65xpected_ledger_size\x18\x04 \x01(\x04\x42\x02\x30\x01\x12\x1c\n\x14ledger_current_index\x18\x05 \x01(\r\x12*\n\x06levels\x18\x06 \x01(\x0b\x32\x1a.org.xrpl.rpc.v1.FeeLevels\x12\x1a\n\x0emax_queue_size\x18\x07 \x01(\x04\x42\x02\x30\x01\"\xdd\x01\n\x03\x46\x65\x65\x12\x31\n\x08\x62\x61se_fee\x18\x01 \x01(\x0b\x32\x1f.org.xrpl.rpc.v1.XRPDropsAmount\x12\x33\n\nmedian_fee\x18\x02 \x01(\x0b\x32\x1f.org.xrpl.rpc.v1.XRPDropsAmount\x12\x34\n\x0bminimum_fee\x18\x03 \x01(\x0b\x32\x1f.org.xrpl.rpc.v1.XRPDropsAmount\x12\x38\n\x0fopen_ledger_fee\x18\x04 \x01(\x0b\x32\x1f.org.xrpl.rpc.v1.XRPDropsAmount\"|\n\tFeeLevels\x12\x18\n\x0cmedian_level\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\x19\n\rminimum_level\x18\x02 \x01(\x04\x42\x02\x30\x01\x12\x1d\n\x11open_ledger_level\x18\x03 \x01(\x04\x42\x02\x30\x01\x12\x1b\n\x0freference_level\x18\x04 \x01(\x04\x42\x02\x30\x01\x42\x13\n\x0forg.xrpl.rpc.v1P\x01\x62\x06proto3'
  ,
  dependencies=[org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2.DESCRIPTOR,])




_GETFEEREQUEST = _descriptor.Descriptor(
  name='GetFeeRequest',
  full_name='org.xrpl.rpc.v1.GetFeeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=80,
  serialized_end=95,
)


_GETFEERESPONSE = _descriptor.Descriptor(
  name='GetFeeResponse',
  full_name='org.xrpl.rpc.v1.GetFeeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='current_ledger_size', full_name='org.xrpl.rpc.v1.GetFeeResponse.current_ledger_size', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='current_queue_size', full_name='org.xrpl.rpc.v1.GetFeeResponse.current_queue_size', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fee', full_name='org.xrpl.rpc.v1.GetFeeResponse.fee', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expected_ledger_size', full_name='org.xrpl.rpc.v1.GetFeeResponse.expected_ledger_size', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ledger_current_index', full_name='org.xrpl.rpc.v1.GetFeeResponse.ledger_current_index', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='levels', full_name='org.xrpl.rpc.v1.GetFeeResponse.levels', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_queue_size', full_name='org.xrpl.rpc.v1.GetFeeResponse.max_queue_size', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=98,
  serialized_end=350,
)


_FEE = _descriptor.Descriptor(
  name='Fee',
  full_name='org.xrpl.rpc.v1.Fee',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='base_fee', full_name='org.xrpl.rpc.v1.Fee.base_fee', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='median_fee', full_name='org.xrpl.rpc.v1.Fee.median_fee', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minimum_fee', full_name='org.xrpl.rpc.v1.Fee.minimum_fee', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='open_ledger_fee', full_name='org.xrpl.rpc.v1.Fee.open_ledger_fee', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=353,
  serialized_end=574,
)


_FEELEVELS = _descriptor.Descriptor(
  name='FeeLevels',
  full_name='org.xrpl.rpc.v1.FeeLevels',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='median_level', full_name='org.xrpl.rpc.v1.FeeLevels.median_level', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minimum_level', full_name='org.xrpl.rpc.v1.FeeLevels.minimum_level', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='open_ledger_level', full_name='org.xrpl.rpc.v1.FeeLevels.open_ledger_level', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reference_level', full_name='org.xrpl.rpc.v1.FeeLevels.reference_level', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=576,
  serialized_end=700,
)

_GETFEERESPONSE.fields_by_name['fee'].message_type = _FEE
_GETFEERESPONSE.fields_by_name['levels'].message_type = _FEELEVELS
_FEE.fields_by_name['base_fee'].message_type = org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2._XRPDROPSAMOUNT
_FEE.fields_by_name['median_fee'].message_type = org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2._XRPDROPSAMOUNT
_FEE.fields_by_name['minimum_fee'].message_type = org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2._XRPDROPSAMOUNT
_FEE.fields_by_name['open_ledger_fee'].message_type = org_dot_xrpl_dot_rpc_dot_v1_dot_amount__pb2._XRPDROPSAMOUNT
DESCRIPTOR.message_types_by_name['GetFeeRequest'] = _GETFEEREQUEST
DESCRIPTOR.message_types_by_name['GetFeeResponse'] = _GETFEERESPONSE
DESCRIPTOR.message_types_by_name['Fee'] = _FEE
DESCRIPTOR.message_types_by_name['FeeLevels'] = _FEELEVELS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetFeeRequest = _reflection.GeneratedProtocolMessageType('GetFeeRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEEREQUEST,
  '__module__' : 'org.xrpl.rpc.v1.get_fee_pb2'
  # @@protoc_insertion_point(class_scope:org.xrpl.rpc.v1.GetFeeRequest)
  })
_sym_db.RegisterMessage(GetFeeRequest)

GetFeeResponse = _reflection.GeneratedProtocolMessageType('GetFeeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEERESPONSE,
  '__module__' : 'org.xrpl.rpc.v1.get_fee_pb2'
  # @@protoc_insertion_point(class_scope:org.xrpl.rpc.v1.GetFeeResponse)
  })
_sym_db.RegisterMessage(GetFeeResponse)

Fee = _reflection.GeneratedProtocolMessageType('Fee', (_message.Message,), {
  'DESCRIPTOR' : _FEE,
  '__module__' : 'org.xrpl.rpc.v1.get_fee_pb2'
  # @@protoc_insertion_point(class_scope:org.xrpl.rpc.v1.Fee)
  })
_sym_db.RegisterMessage(Fee)

FeeLevels = _reflection.GeneratedProtocolMessageType('FeeLevels', (_message.Message,), {
  'DESCRIPTOR' : _FEELEVELS,
  '__module__' : 'org.xrpl.rpc.v1.get_fee_pb2'
  # @@protoc_insertion_point(class_scope:org.xrpl.rpc.v1.FeeLevels)
  })
_sym_db.RegisterMessage(FeeLevels)


DESCRIPTOR._options = None
_GETFEERESPONSE.fields_by_name['current_ledger_size']._options = None
_GETFEERESPONSE.fields_by_name['current_queue_size']._options = None
_GETFEERESPONSE.fields_by_name['expected_ledger_size']._options = None
_GETFEERESPONSE.fields_by_name['max_queue_size']._options = None
_FEELEVELS.fields_by_name['median_level']._options = None
_FEELEVELS.fields_by_name['minimum_level']._options = None
_FEELEVELS.fields_by_name['open_ledger_level']._options = None
_FEELEVELS.fields_by_name['reference_level']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: org/xrpl/rpc/v1/account.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='org/xrpl/rpc/v1/account.proto',
  package='org.xrpl.rpc.v1',
  syntax='proto3',
  serialized_options=b'\n\017org.xrpl.rpc.v1P\001',
  serialized_pb=b'\n\x1dorg/xrpl/rpc/v1/account.proto\x12\x0forg.xrpl.rpc.v1\"!\n\x0e\x41\x63\x63ountAddress\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x13\n\x0forg.xrpl.rpc.v1P\x01\x62\x06proto3'
)




_ACCOUNTADDRESS = _descriptor.Descriptor(
  name='AccountAddress',
  full_name='org.xrpl.rpc.v1.AccountAddress',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='org.xrpl.rpc.v1.AccountAddress.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=83,
)

DESCRIPTOR.message_types_by_name['AccountAddress'] = _ACCOUNTADDRESS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AccountAddress = _reflection.GeneratedProtocolMessageType('AccountAddress', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTADDRESS,
  '__module__' : 'org.xrpl.rpc.v1.account_pb2'
  # @@protoc_insertion_point(class_scope:org.xrpl.rpc.v1.AccountAddress)
  })
_sym_db.RegisterMessage(AccountAddress)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import typing as t

from xpring.hashes import sha512half
from xpring.key_pair import KeyPair
from xpring.serialization import (
    PREFIX_TRANSACTION_ID,
    PREFIX_TRANSACTION_SIGNATURE,
    serialize_transaction,
)
from xpring.algorithms.signing import SigningAlgorithm
from xpring.types import (
    AccountId,
    Address,
    EncodedSeed,
    Seed,
    PrivateKey,
    PublicKey,
    Signature,
    SignedTransaction,
    Transaction,
)


class Wallet:

    def __init__(self, key_pair: KeyPair):
        self.key_pair = key_pair

    @classmethod
    def from_seed(cls, seed: EncodedSeed):
        key_pair = KeyPair.from_encoded_seed(seed)
        return cls(key_pair)

    @property
    def seed(self) -> Seed:
        return self.key_pair.seed

    @property
    def algorithm(self) -> SigningAlgorithm:
        return self.key_pair.algorithm

    @property
    def account_id(self) -> AccountId:
        return self.key_pair.account_id

    @property
    def address(self) -> Address:
        return self.key_pair.address

    @property
    def public_key(self) -> PublicKey:
        return self.key_pair.public_key

    @property
    def private_key(self) -> PrivateKey:
        return self.key_pair.private_key

    def sign(self, message: bytes) -> Signature:
        return self.key_pair.sign(message)

    def sign_transaction(self, transaction: Transaction) -> SignedTransaction:
        result = {**transaction, 'SigningPubKey': self.public_key.hex().upper()}
        blob = serialize_transaction(result, signing=True)
        signature = self.sign(PREFIX_TRANSACTION_SIGNATURE + blob)
        result['TxnSignature'] = signature.hex().upper()
        blob = serialize_transaction(result)
        digest = sha512half(PREFIX_TRANSACTION_ID + blob)
        result['hash'] = digest.hex().upper()
        return result

    def verify(self, message: bytes, signature: bytes) -> bool:
        return self.key_pair.verify(message, t.cast(Signature, signature))



import hashlib

import nacl.encoding
import nacl.hash


def sha256(bites: bytes) -> bytes:
    return nacl.hash.sha256(bites, encoder=nacl.encoding.RawEncoder)


def checksum(bites: bytes) -> bytes:
    return sha256(sha256(bites))[:4]


def sha512half(bites: bytes) -> bytes:
    return nacl.hash.sha512(bites, encoder=nacl.encoding.RawEncoder)[:32]


def ripemd160(bites: bytes) -> bytes:
    hasher = hashlib.new('ripemd160')
    hasher.update(bites)
    return hasher.digest()


class IdentityHash:
    digest_size = 32
    block_size = 32
    name = 'identity'

    def __init__(self, data: bytes = b'') -> None:
        self.data = bytes(data)

    def update(self, data: bytes) -> None:
        self.data += bytes(data)

    def digest(self) -> bytes:
        return self.data

    def hexdigest(self) -> str:
        return self.digest().hex()

    def copy(self) -> 'IdentityHash':
        return self.__class__(self.data)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import get_account_info_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2
from . import get_account_transaction_history_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2
from . import get_fee_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2
from . import get_transaction_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2
from . import submit_pb2 as org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2


class XRPLedgerAPIServiceStub(object):
    """RPCs available to interact with the XRP Ledger.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetAccountInfo = channel.unary_unary(
                '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetAccountInfo',
                request_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoRequest.SerializeToString,
                response_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoResponse.FromString,
                )
        self.GetFee = channel.unary_unary(
                '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetFee',
                request_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeRequest.SerializeToString,
                response_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeResponse.FromString,
                )
        self.SubmitTransaction = channel.unary_unary(
                '/org.xrpl.rpc.v1.XRPLedgerAPIService/SubmitTransaction',
                request_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionRequest.SerializeToString,
                response_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionResponse.FromString,
                )
        self.GetTransaction = channel.unary_unary(
                '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetTransaction',
                request_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionRequest.SerializeToString,
                response_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionResponse.FromString,
                )
        self.GetAccountTransactionHistory = channel.unary_unary(
                '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetAccountTransactionHistory',
                request_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryRequest.SerializeToString,
                response_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryResponse.FromString,
                )


class XRPLedgerAPIServiceServicer(object):
    """RPCs available to interact with the XRP Ledger.
    """

    def GetAccountInfo(self, request, context):
        """Get account info for an account on the XRP Ledger.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFee(self, request, context):
        """Get the fee for a transaction on the XRP Ledger.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubmitTransaction(self, request, context):
        """Submit a signed transaction to the XRP Ledger.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTransaction(self, request, context):
        """Get the status of a transaction
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAccountTransactionHistory(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_XRPLedgerAPIServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetAccountInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAccountInfo,
                    request_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoRequest.FromString,
                    response_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoResponse.SerializeToString,
            ),
            'GetFee': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFee,
                    request_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeRequest.FromString,
                    response_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeResponse.SerializeToString,
            ),
            'SubmitTransaction': grpc.unary_unary_rpc_method_handler(
                    servicer.SubmitTransaction,
                    request_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionRequest.FromString,
                    response_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionResponse.SerializeToString,
            ),
            'GetTransaction': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTransaction,
                    request_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionRequest.FromString,
                    response_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionResponse.SerializeToString,
            ),
            'GetAccountTransactionHistory': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAccountTransactionHistory,
                    request_deserializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryRequest.FromString,
                    response_serializer=org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'org.xrpl.rpc.v1.XRPLedgerAPIService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class XRPLedgerAPIService(object):
    """RPCs available to interact with the XRP Ledger.
    """

    @staticmethod
    def GetAccountInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetAccountInfo',
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoRequest.SerializeToString,
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__info__pb2.GetAccountInfoResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFee(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetFee',
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeRequest.SerializeToString,
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__fee__pb2.GetFeeResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SubmitTransaction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.xrpl.rpc.v1.XRPLedgerAPIService/SubmitTransaction',
            org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionRequest.SerializeToString,
            org_dot_xrpl_dot_rpc_dot_v1_dot_submit__pb2.SubmitTransactionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTransaction(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetTransaction',
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionRequest.SerializeToString,
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__transaction__pb2.GetTransactionResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAccountTransactionHistory(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/org.xrpl.rpc.v1.XRPLedgerAPIService/GetAccountTransactionHistory',
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryRequest.SerializeToString,
            org_dot_xrpl_dot_rpc_dot_v1_dot_get__account__transaction__history__pb2.GetAccountTransactionHistoryResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# @generated by generate_proto_mypy_stubs.py.  Do not edit!
import sys
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from .account_pb2 import (
    AccountAddress as org___xrpl___rpc___v1___account_pb2___AccountAddress,
)

from .amount_pb2 import (
    Currency as org___xrpl___rpc___v1___amount_pb2___Currency,
    CurrencyAmount as org___xrpl___rpc___v1___amount_pb2___CurrencyAmount,
)

from typing import (
    Optional as typing___Optional,
    Text as typing___Text,
    Union as typing___Union,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int
if sys.version_info < (3,):
    builtin___buffer = buffer
    builtin___unicode = unicode


class CancelAfter(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> CancelAfter: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> CancelAfter: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class ClearFlag(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ClearFlag: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ClearFlag: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class CloseTime(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> CloseTime: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> CloseTime: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Date(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Date: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Date: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class DestinationTag(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DestinationTag: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DestinationTag: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Expiration(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Expiration: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Expiration: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class FinishAfter(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> FinishAfter: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> FinishAfter: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Flags(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Flags: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Flags: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class HighQualityIn(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HighQualityIn: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HighQualityIn: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class HighQualityOut(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HighQualityOut: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HighQualityOut: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LastLedgerSequence(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LastLedgerSequence: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LastLedgerSequence: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LowQualityIn(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LowQualityIn: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LowQualityIn: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LowQualityOut(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LowQualityOut: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LowQualityOut: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class OfferSequence(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> OfferSequence: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> OfferSequence: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class OwnerCount(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> OwnerCount: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> OwnerCount: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class PreviousTransactionLedgerSequence(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PreviousTransactionLedgerSequence: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PreviousTransactionLedgerSequence: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class QualityIn(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> QualityIn: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> QualityIn: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class QualityOut(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> QualityOut: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> QualityOut: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class ReferenceFeeUnits(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ReferenceFeeUnits: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ReferenceFeeUnits: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class ReserveBase(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ReserveBase: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ReserveBase: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class ReserveIncrement(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> ReserveIncrement: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> ReserveIncrement: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Sequence(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Sequence: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Sequence: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SetFlag(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SetFlag: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SetFlag: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SettleDelay(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SettleDelay: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SettleDelay: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SignerListID(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SignerListID: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SignerListID: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SignerQuorum(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SignerQuorum: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SignerQuorum: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SignerWeight(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SignerWeight: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SignerWeight: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SourceTag(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SourceTag: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SourceTag: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TickSize(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TickSize: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TickSize: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TransferRate(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TransferRate: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TransferRate: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class BaseFee(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BaseFee: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BaseFee: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class BookNode(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BookNode: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BookNode: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class DestinationNode(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DestinationNode: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DestinationNode: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class HighNode(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HighNode: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HighNode: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class IndexNext(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> IndexNext: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> IndexNext: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class IndexPrevious(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> IndexPrevious: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> IndexPrevious: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LowNode(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LowNode: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LowNode: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class OwnerNode(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___int

    def __init__(self,
        *,
        value : typing___Optional[builtin___int] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> OwnerNode: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> OwnerNode: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class EmailHash(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> EmailHash: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> EmailHash: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerGetsIssuer(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerGetsIssuer: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerGetsIssuer: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerPaysIssuer(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerPaysIssuer: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerPaysIssuer: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class AccountTransactionID(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> AccountTransactionID: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> AccountTransactionID: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class BookDirectory(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> BookDirectory: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> BookDirectory: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Channel(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Channel: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Channel: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class CheckID(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> CheckID: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> CheckID: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Hash(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Hash: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Hash: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Index(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Index: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Index: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class InvoiceID(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> InvoiceID: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> InvoiceID: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class PreviousTransactionID(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PreviousTransactionID: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PreviousTransactionID: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class RootIndex(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> RootIndex: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> RootIndex: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Condition(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Condition: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Condition: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Fulfillment(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Fulfillment: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Fulfillment: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class MemoData(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MemoData: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MemoData: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class MemoFormat(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MemoFormat: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MemoFormat: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class MemoType(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MemoType: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MemoType: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class MessageKey(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> MessageKey: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> MessageKey: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class PublicKey(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PublicKey: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PublicKey: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class PaymentChannelSignature(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> PaymentChannelSignature: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> PaymentChannelSignature: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SigningPublicKey(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SigningPublicKey: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SigningPublicKey: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TransactionSignature(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: builtin___bytes

    def __init__(self,
        *,
        value : typing___Optional[builtin___bytes] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TransactionSignature: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TransactionSignature: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerGetsCurreny(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___Currency: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___Currency] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerGetsCurreny: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerGetsCurreny: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerPaysCurrency(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___Currency: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___Currency] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerPaysCurrency: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerPaysCurrency: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Amount(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Amount: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Amount: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Balance(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Balance: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Balance: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class DeliverMin(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeliverMin: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeliverMin: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class DeliveredAmount(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> DeliveredAmount: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> DeliveredAmount: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class HighLimit(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> HighLimit: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> HighLimit: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LimitAmount(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LimitAmount: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LimitAmount: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class LowLimit(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> LowLimit: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> LowLimit: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SendMax(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SendMax: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SendMax: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerGets(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerGets: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerGets: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class TakerPays(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___amount_pb2___CurrencyAmount: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___amount_pb2___CurrencyAmount] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> TakerPays: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> TakerPays: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Account(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Account: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Account: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Authorize(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Authorize: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Authorize: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Destination(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Destination: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Destination: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Owner(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Owner: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Owner: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class RegularKey(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> RegularKey: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> RegularKey: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Unauthorize(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def value(self) -> org___xrpl___rpc___v1___account_pb2___AccountAddress: ...

    def __init__(self,
        *,
        value : typing___Optional[org___xrpl___rpc___v1___account_pb2___AccountAddress] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Unauthorize: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Unauthorize: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class Domain(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    value = ... # type: typing___Text

    def __init__(self,
        *,
        value : typing___Optional[typing___Text] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> Domain: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> Domain: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def ClearField(self, field_name: typing_extensions___Literal[u"value"]) -> None: ...
    else:
        def ClearField(self, field_name: typing_extensions___Literal[u"value",b"value"]) -> None: ...

class SignerEntry(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def account(self) -> Account: ...

    @property
    def signer_weight(self) -> SignerWeight: ...

    def __init__(self,
        *,
        account : typing___Optional[Account] = None,
        signer_weight : typing___Optional[SignerWeight] = None,
        ) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def FromString(cls, s: builtin___bytes) -> SignerEntry: ...
    else:
        @classmethod
        def FromString(cls, s: typing___Union[builtin___bytes, builtin___buffer, builtin___unicode]) -> SignerEntry: ...
    def MergeFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    def CopyFrom(self, other_msg: google___protobuf___message___Message) -> None: ...
    if sys.version_info >= (3,):
        def HasField(self, field_name: typing_extensions___Literal[u"account",u"signer_weight"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"account",u"signer_weight"]) -> None: ...
    else:
        def HasField(self, field_name: typing_extensions___Literal[u"account",b"account",u"signer_weight",b"signer_weight"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"account",b"account",u"signer_weight",b"signer_weight"]) -> None: ...
